@@ -77,9 +77,6 @@ for(f=0;f<strlen(str);f++)
       vga_drawline(a,b,a,b+s4);
     }
 
-  /* Hmmm? Why are you still here, exactly? I did tell you not to look at
-   * this, remember. Some people just don't listen. Well, I deny everything.
-   */
 
   /*** 2nd step: fill in rest - this is the *really* long, messy bit :) ***/
   
@@ -277,8 +274,18 @@ for(f=0;f<strlen(str);f++)
     case ')':
       fontc_ur(a,b+s1,siz); fontc_lr(a,b+s3,siz);
       vga_drawline(a+s1,b+s1,a+s1,b+s3); break;
+    case '%':
+      vga_drawpixel(a,b);
+      vga_drawpixel(a+s2,b+s4);
+      /* FALLS THROUGH drawing the slash to finish the % */
     case '/':
       vga_drawline(a,b+s4,a+s2,b); break;
+    case '?':
+      fontc_u(a+s1,b+s1,siz);
+      vga_drawline(a+s2,b+s1,a+s1,b+s2);
+      vga_drawline(a+s1,b+s2,a+s1,b+s3);
+      vga_drawpixel(a+s1,b+s4);
+      break;
     }
 
     
@@ -288,7 +295,7 @@ for(f=0;f<strlen(str);f++)
     a+=s1;
   else
     {
-    if(index("/abcdeghnopqrsuvxyzABCDEFGHIJKLNOPQRSTUVXYZ0123456789",c)!=NULL)
+    if(index("?/%abcdeghnopqrsuvxyzABCDEFGHIJKLNOPQRSTUVXYZ0123456789",c)!=NULL)
       a+=s2;
     else
       {
@@ -399,14 +406,6 @@ vga_drawline(x+r,y,x+r34,y+r34);
 vga_drawline(x+r34,y+r34,x,y+r);
 }
 
-/* still here? Oh dear. Hmm, ok, time for some music perhaps?
- * "All the same, we take our chances
- *  Laughed at by time, tricked by circumstances
- *  'Plus ca change plus c'est la meme chose'
- *  The more that things change...
- *  The more they stay the same." - the Puhfesser (on the drumkit)
- */
-
 
 /* here's a new one - this gets how wide text will be */
 int vgatextsize(sizearg,str)
