@@ -10,6 +10,7 @@
 #include <unistd.h>
 #include <malloc.h>
 #include <stdlib.h>
+#include "zgv.h"
 #include "gifeng.h"
 
 
@@ -72,7 +73,7 @@ struct {
 
 /* prototypes */
 int readgif(char *,byte **,byte **,hffunc);
-void getgifinfo(GIFINFO *);
+void getgifinfo(PICINFO *);
 int readgifhed(FILE *);
 int twopower(int);
 void readcolmap(FILE *,byte *);
@@ -91,7 +92,7 @@ void dummyfunc(int,int);
  *   result=readgif("thingy.gif",&ptr_to_image_data,&ptr_to_palette_data,
                                  show_how_far_func);
  * show_how_far_func, for percent complete displays etc., can be NULL.
- * any value of 'result' other than _GIF_OK means something went wrong,
+ * any value of 'result' other than _PIC_OK means something went wrong,
  *  see gifeng.h for more details and error codes.
  */
 int readgif(giffn,theimageptr,palptr,howfarfunc)
@@ -113,16 +114,16 @@ else
      /* besides, local variables are for pencil-necked Modula-2 programmers */
 
 if((in=global_gif_infile=fopen(giffn,"rb"))==NULL)
-  return(_GIFERR_NOFILE);
+  return(_PICERR_NOFILE);
 else
   {
   if((palette=global_palette_ptr=(byte *)malloc(768))==NULL)
     {
     fclose(in);
-    return(_GIFERR_NOMEM);
+    return(_PICERR_NOMEM);
     }
 
-  if((poserr=  readgifhed(in)  )!=_GIF_OK)
+  if((poserr=  readgifhed(in)  )!=_PIC_OK)
     {
     fclose(in);
     free(palette);
@@ -131,7 +132,7 @@ else
     
   readcolmap(in,palette);
                
-  if((poserr=  readimagehed(in))!=_GIF_OK) 
+  if((poserr=  readimagehed(in))!=_PIC_OK) 
     {
     fclose(in);
     free(palette);
@@ -142,16 +143,16 @@ else
     {
     fclose(in);
     free(palette);
-    return(_GIFERR_NOMEM);
+    return(_PICERR_NOMEM);
     }
 
-  if( decompress(in) !=_GIF_OK) return(_GIFERR_CORRUPT);
+  if( decompress(in) !=_PIC_OK) return(_PICERR_CORRUPT);
                
   fclose(in);
   *theimageptr=image;
   *palptr=palette;
 
-  return(_GIF_OK);
+  return(_PIC_OK);
   }
 }
 
@@ -164,7 +165,7 @@ int a,b;
 
 
 void getgifinfo(ginfo)
-GIFINFO *ginfo;
+PICINFO *ginfo;
 {
 ginfo->width=width;
 ginfo->height=height;
@@ -181,12 +182,12 @@ int f;
 
 fread(&gifhed,sizeof(gifhed),1,in);
 if(strncmp(gifhed.sig,"GIF",3))
-  return(_GIFERR_NOTGIFFILE);
+  return(_PICERR_BADMAGIC);
 if((gifhed.misc&128)==0)
-  return(_GIFERR_NOCOLOURMAP);
+  return(_PICERR_NOCOLOURMAP);
 bpp=(gifhed.misc&7)+1;
 numcols=twopower(bpp);
-return(_GIF_OK);
+return(_PIC_OK);
 }
 
 
@@ -231,11 +232,11 @@ while(c=='!')      /* oh damn it, they've put an ext. block in, ditch it */
   c=fgetc(in);     /* test for image again */
   }
 if(c!=',')
-  return(_GIFERR_NOIMAGE);
+  return(_PICERR_NOIMAGE);
 fread(&imagehed,sizeof(imagehed),1,in);
 
 if((imagehed.misc&128)!=0)
-  return(_GIFERR_LOCALMAP);
+  return(_PICERR_LOCALMAP);
 if((imagehed.misc&64)!=0)
   {
   interlaced=1;
@@ -247,7 +248,7 @@ else
   interlaced=0;
 width=(imagehed.wide_lo+256*imagehed.wide_hi);
 height=(imagehed.high_lo+256*imagehed.high_hi);
-return(_GIF_OK);
+return(_PIC_OK);
 }
 
 
@@ -268,7 +269,7 @@ inittable();
 
 do
   {
-  if(!readcode(&newcode,csize,in)) return(_GIFERR_CORRUPT);
+  if(!readcode(&newcode,csize,in)) return(_PICERR_CORRUPT);
   oldcode=newcode;
   }
 while(newcode==dc_cc);
@@ -276,14 +277,14 @@ outputstring(newcode);
 
 while((newcode!=dc_eoi)&&(!stoprightnow))
   {
-  if(!readcode(&newcode,csize,in)) return(_GIFERR_CORRUPT);
+  if(!readcode(&newcode,csize,in)) return(_PICERR_CORRUPT);
   if(newcode!=dc_eoi)
     {
     if(newcode==dc_cc)
       {
       inittable();
       csize=orgcsize;
-      if(!readcode(&newcode,csize,in)) return(_GIFERR_CORRUPT);
+      if(!readcode(&newcode,csize,in)) return(_PICERR_CORRUPT);
       oldcode=newcode;
       outputstring(newcode);
       }
@@ -311,7 +312,7 @@ while((newcode!=dc_eoi)&&(!stoprightnow))
     }
   }
 
-return(_GIF_OK);
+return(_PIC_OK);
 }
 
 
