@@ -1,4 +1,4 @@
-/* ZGV v1.3 - (c) 1993 Russell Marks for improbabledesigns.
+/* ZGV v2.0 - (c) 1993,1994 Russell Marks for improbabledesigns.
  * See README for license details.
  *
  * helppage.c - for the online help page displays.
@@ -11,19 +11,22 @@
 #include "font.h"
 #include "readnbkey.h"
 #include "handlevt.h"
+#include "rc_config.h"
+#include "rcfile.h"
 
 
 
-/* we go to text mode to workaround svgalib bug */
 helpscreenon()
 {
-vga_setmode(TEXT);
+/* aha!!! Finally diked this bastard out */
+/* vga_setmode(TEXT); */
 vga_setmode(G640x480x16);
 vga_setcolor(15);
-vga_setpalette(0,30,30,30);
-vga_setpalette(1,20,20,20);
-vga_setpalette(2,40,40,40);
-vga_setpalette(15,0,0,0);
+
+vga_setpalette(0,cfg.medium_r,cfg.medium_g,cfg.medium_b);
+vga_setpalette(1,cfg.dark_r,cfg.dark_g,cfg.dark_b);
+vga_setpalette(2,cfg.light_r,cfg.light_g,cfg.light_b);
+vga_setpalette(15,cfg.black_r,cfg.black_g,cfg.black_b);
 
 draw3dbox(0,0,639,479,2,1, 2,1);
 draw3dbox(10,10,629,469,1,0, 2,1);
@@ -70,5 +73,5 @@ while(!quit)
     usleep(10000);
     }
   }
-vga_setmode(TEXT);
+/* vga_setmode(TEXT);    hmm... should this still be here? */
 }
