@@ -7,6 +7,13 @@
 
 struct zgv_config {
   int videomode,zoom,vkludge,jpeg24bit;
+  int centreflag,blockcursor,thicktext;
+  int betterpgm;  /* grinds PGM to 24-bit internally allowing 256 shades */
+  int hicolmodes;	/* force zgv to think card has high-colour modes */
+  int nodelprompt,perfectindex;
+  int xvpic_index;
+  int onefile_progress,cleartext;
+  
   int brightness;
   double contrast;
   int black_r,black_g,black_b;
