@@ -10,3 +10,5 @@ extern struct zgv_config cfg;
 extern int getconfig(void);
 extern int saveconfig(void);
 extern int modematch(int,int,int);
+extern int parsecommandline(int argc,char **argv);
+extern int usage_help();
