@@ -10,11 +10,12 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <stdlib.h>
 #include <ctype.h>
 #include <sys/file.h>
+#include "zgv.h"
 #include "gifeng.h"
 #include "usejpeg.h"
+#include "resizepic.h"
 #include "tnpicfont.h"
 
 
@@ -22,7 +23,7 @@
 #define TEXTHI 22
 
 int vkludge=1,pixelsize=1;
-
+int idx_dark,idx_light,idx_black;	/* XXX ***KLUDGE*** */
 
 int jpeg_quality=70;
 
@@ -30,11 +31,9 @@ byte *theimage,*palrgb,*redptr,*greenptr,*blueptr;
 int outwidth,outheight;
 int width,height,numcols;
 int scrnwide=128,scrnhigh=96;
-int palr[256],palg[256],palb[256]; 
 
 
 int readgiforjpg(char *,hffunc);
-byte *resizepic();
 
 
 
@@ -45,6 +44,7 @@ char **argv;
 byte *resizedrgb,*outimage;
 char str[1024],*org;
 int f,total,cx,cy,xsiz,ysiz;
+int tmpw,tmph;
 div_t cxy;
 
 if(argc>=3)
@@ -90,7 +90,7 @@ else
     {
     printf("Reading %s...\n",argv[f+2]);
     cxy=div(f,PICS_ACROSS);
-    if(readgiforjpg(argv[f+2],NULL)!=0)
+    if(readgiforjpg(argv[f+2],NULL)!=_PIC_OK)
       {
       /* JPEG errors are reported automatically...
        * it'll be a GIF if we get here.
@@ -98,7 +98,10 @@ else
       printf("Error reading GIF file.\n");
       exit(1);
       }
-    resizedrgb=resizepic();
+      
+    tmpw=scrnwide; tmph=scrnhigh;
+    resizedrgb=resizepic(theimage,palrgb,palrgb+256,palrgb+512,
+    					width,height,&tmpw,&tmph);
     addpic(outimage,resizedrgb,(cxy.rem)*scrnwide,
     		(cxy.quot)*(scrnhigh+TEXTHI),xsiz);
     if(strrchr(argv[f+2],'/')!=NULL)
@@ -200,7 +203,7 @@ char *giffn;
 hffunc howfarfunc;
 {
 int result,isjpeg;
-GIFINFO ginfo;
+PICINFO ginfo;
 byte *palette;
 
 if(!strcmp(strrchr(giffn,'.'),".jpg"))
@@ -210,15 +213,15 @@ if(!strcmp(strrchr(giffn,'.'),".jpg"))
   theimage=NULL;
   result=read_JPEG_file(giffn,howfarfunc,&palette);
   if(theimage==NULL)
-    result=0;
+    result=_PICERR_CORRUPT;
   else
-    if(result==0) free(theimage);
+    if(result!=_PIC_OK) free(theimage);
   }
 else
   {
   isjpeg=0;
   result=readgif(giffn,&theimage,&palette,howfarfunc);
-  if(result==_GIF_OK)
+  if(result==_PIC_OK)
     {
     getgifinfo(&ginfo);
     height=ginfo.height; width=ginfo.width;
@@ -227,120 +230,12 @@ else
     return(result);
   }
 numcols=ginfo.numcols;
-if(((isjpeg)&&(result==1))||((!isjpeg)&&(result==_GIF_OK)))
+if(result==_PIC_OK)
   {
   palrgb=palette;
-  filterpal(palrgb);
   /********** all memory still allocated **********/
   /* (we lose it later on) */
   }
-/* when is a result not a result? */
-if(isjpeg) result=!result;
+
 return(result);
 }
-
-
-filterpal(palette)
-byte *palette;
-{
-int f;
-
-for(f=0;f<256;f++)   /* don't *really* need to know number of colours */
-  {
-  palr[f]=palette[    f];
-  palg[f]=palette[256+f];
-  palb[f]=palette[512+f];
-  }
-}
-
-
-
-
-/* this routine is nasty writ big, but about as quick as I can manage */
-byte *resizepic()
-{
-register int a,b,x,yp,yw;
-long y,sw,sh,lastyp;
-int xoff,yoff;
-int c,pixwide,pixhigh,pr,pg,pb;
-long place;
-int bigimage;
-byte *rline;
-int tmp1,tmp2,tr,tg,tb,tn;
-int xypos;
-
-if((rline=malloc(scrnwide*scrnhigh*3))==NULL) return(0);
-for(x=0;x<(scrnwide*scrnhigh*3);x++)
-  rline[x]=0;
-
-/* try landscapey */
-sw=scrnwide; sh=(int)((scrnwide*((long)height))/((long)width));
-if(sh>scrnhigh)
-  /* no, oh well portraity then */
-  { sh=scrnhigh; sw=(int)((scrnhigh*((long)width))/((long)height)); }
-
-/* so now our zoomed image will be sw x sh */
-bigimage=(width>sw)?1:0;   /* 1 if image has been reduced, 0 if made bigger */
-if(bigimage)
-  /* it's been reduced - easy, just make 'em fit in less space */
-  {
-  lastyp=-1;
-  pixhigh=(int)(((float)height)/((float)sh)+0.5);
-  pixwide=(int)(((float)width)/((float)sw)+0.5);
-  /*if(pixhigh==0)*/ pixhigh++;
-  /*if(pixwide==0)*/ pixwide++;
-  for(y=0;y<height;y++)
-    {
-    yp=(y*sh)/height;
-    if(yp!=lastyp)
-      {
-      yw=y*width;
-      if(vkludge) /* we try to resample a bit. get that pentium RSN :) */
-        {
-        for(x=0;x<width;x++,yw++)
-          {
-          tr=tg=tb=tn=0;
-          for(b=0;(b<pixhigh)&&(y+b<height);b++)
-            for(a=0;(a<pixwide)&&(x+a<width);a++)
-              {
-              tmp2=*(theimage+yw+a+b*width);
-              tr+=palr[tmp2];
-              tg+=palg[tmp2];
-              tb+=palb[tmp2];
-              tn++;
-              }
-          tr/=tn; tg/=tn; tb/=tn;
-          xypos=3*(((x*sw)/width)+yp*scrnwide);
-          rline[xypos]=tr;
-          rline[xypos+1]=tg;
-          rline[xypos+2]=tb;
-          }
-        }
-      else
-        for(x=0;x<width;x++,yw++)
-          {
-          tmp2=*(theimage+yw);
-          xypos=3*(((x*sw)/width)+yp*scrnwide);
-          rline[xypos  ]=palr[tmp2];
-          rline[xypos+1]=palg[tmp2];
-          rline[xypos+2]=palb[tmp2];
-          }
-      lastyp=yp;
-      }
-    }
-  }
-else
-  /* we just leave it the same size */
-  {
-  for(y=0;y<height;y++)
-    for(x=0;x<width;x++)
-      {
-      c=theimage[y*width+x];
-      rline[3*(y*scrnwide+x)  ]=palr[c];
-      rline[3*(y*scrnwide+x)+1]=palg[c];
-      rline[3*(y*scrnwide+x)+2]=palb[c];
-      }
-  }
-  
-return(rline);
-}
