@@ -9,7 +9,7 @@
 #include <sys/file.h>  /* for open et al */
 #include "3deffects.h"
 #include <jinclude.h>
-#include "gifeng.h"
+#include "zgv.h"
 #include "usejpeg.h"
 #include "vgadisp.h"   /* for pixelsize */
 
@@ -36,7 +36,7 @@ static external_methods_ptr emethods; /* needed for access to message_parm */
 
 /* error print routine */
 /* XXX we need to somehow use printf instead if we're in text mode,
- * i.e. running gifview (or tnpic?).
+ * i.e. running tnpic.
  */
 
 METHODDEF void
@@ -53,9 +53,9 @@ sprintf(buf1,buf2,
 	  emethods->message_parm[4], emethods->message_parm[5],
 	  emethods->message_parm[6], emethods->message_parm[7]);
 
-if((ttyfd=open("/dev/tty0",O_RDONLY|O_NONBLOCK))>=0)
+if((ttyfd=open("/dev/tty",O_RDONLY|O_NONBLOCK))>=0)
   {
-  msgbox(ttyfd,buf1,MSGBOXTYPE_OK, 2,1,15); /* we assume colours are ok! */
+  msgbox(ttyfd,buf1,MSGBOXTYPE_OK, idx_light,idx_dark,idx_black);
   close(ttyfd);
   }
 
@@ -160,11 +160,23 @@ if(theimage!=NULL)
     else
       {
       for (col = 0; col < cinfo->image_width; col++)
-        {
-        *(theimage+3*(width*imagey+col)+2)=GETJSAMPLE(*ptr0); ptr0++;
-        *(theimage+3*(width*imagey+col)+1)=GETJSAMPLE(*ptr1); ptr1++;
-        *(theimage+3*(width*imagey+col)  )=GETJSAMPLE(*ptr2); ptr2++;
-        }
+        /* this is a bit of a kludge, should really load as 8-bit
+         * and fix on-the-fly in vgadisp.c when displaying, to
+         * save memory.
+         */
+        if(cinfo->jpeg_color_space==CS_GRAYSCALE)
+          {
+          *(theimage+3*(width*imagey+col)  )=GETJSAMPLE(*ptr0);
+          *(theimage+3*(width*imagey+col)+1)=GETJSAMPLE(*ptr0);
+          *(theimage+3*(width*imagey+col)+2)=GETJSAMPLE(*ptr0);
+          ptr0++;
+          }
+        else
+          {
+          *(theimage+3*(width*imagey+col)+2)=GETJSAMPLE(*ptr0); ptr0++;
+          *(theimage+3*(width*imagey+col)+1)=GETJSAMPLE(*ptr1); ptr1++;
+          *(theimage+3*(width*imagey+col)  )=GETJSAMPLE(*ptr2); ptr2++;
+          }
       }
     imagey++;
     if(howfar!=NULL) howfar(imagey,height);
@@ -182,7 +194,7 @@ METHODDEF void
 d_ui_method_selection (decompress_info_ptr cinfo)
 {
   /* fix to greys if greyscale - this is required to read greyscale JPEGs */
-  if((cinfo->jpeg_color_space==CS_GRAYSCALE)&&(pixelsize==1))
+  if(cinfo->jpeg_color_space==CS_GRAYSCALE)
     {
     cinfo->out_color_space=CS_GRAYSCALE;
     cinfo->two_pass_quantize=FALSE;
@@ -209,12 +221,12 @@ byte **palette;
   imagex=imagey=0;
   howfar=howfarfunc;
   if(((*palette)=(byte *)malloc(768))==NULL)
-    return(0);
+    return(_PICERR_NOMEM);
   pal=*palette;
 
 
   if ((cinfo.input_file = global_jpeg_infile = fopen(filename, "rb")) == NULL)
-    return 0;
+    return(_PICERR_NOFILE);
 
   cinfo.output_file = NULL;	/* if no actual output file involved */
   
@@ -243,7 +255,7 @@ byte **palette;
      */
     fclose(cinfo.input_file);
     free(pal);
-    return 0;
+    return(_PICERR_CORRUPT);	/* could actually be anything... */
     }
   
 
@@ -270,5 +282,5 @@ byte **palette;
 
   fclose(cinfo.input_file);
 
-  return 1;			/* indicate success */
+  return(_PIC_OK);
 }
