@@ -1,4 +1,4 @@
-/* ZGV v2.0 - (c) 1993,1994 Russell Marks for improbabledesigns.
+/* ZGV v2.2 - (c) 1993,1994 Russell Marks for improbabledesigns.
  * See README for license details.
  *
  * vgadisp.c - vga specific display routines.
@@ -14,8 +14,11 @@
 #include <sys/file.h>
 #include <vga.h>
 #include <vgagl.h>
+#include "zgv.h"
+#include "magic.h"
 #include "gifeng.h"
 #include "usejpeg.h"
+#include "readpnm.h"
 #include "readnbkey.h"
 #include "handlevt.h"
 #include "helppage.h"
@@ -29,32 +32,33 @@
 int curvgamode;
 int zoom,virtual;
 int vkludge;
-int pixelsize;     /* in bytes - 1 for 8-bit, 3 for 24-bit */
+int pixelsize;		/* in bytes - 1 for 8-bit, 3 for 24-bit */
 static int isjpeg;
 
 char viewerhelp[][80]={
-  "Question mark \t this help page\0",
-  "1 to 9 \t\t mode change\0",
-#ifndef HANDS_ON_VC_CODE    /* ...then we're using the fkeys for svga modes */
-  "F1 - F10  SF1 - SF10 \t SVGA mode change (try these first)\0",
+  "? (question mark) \t this help page",
+  "5 to 8 \t\t mode change",
+#ifndef HANDS_ON_VC_CODE
+  "F1 - F12  SF3 - SF10 \t SVGA mode change (try these first)",
 #endif
-  "0 \t\t\t toggle virtual mode\0",
-  "v \t\t\t toggle smoothing in virtual modes\0",
-  "comma / dot \t contrast down/up\0",
-  "left / right chevron \t brightness down/up\0",
-  "asterisk \t\t reset contrast and brightness\0",
-  "z \t\t\t toggle zoomed mode\0",
-  "r \t\t\t rotate clockwise 90 degs\0",
-  "m \t\t\t mirror\0",
-  "f \t\t\t flip\0",
+  "0 \t\t\t toggle virtual mode",
+  "v \t\t\t toggle smoothing in virtual/zoom modes",
+  "comma / dot \t contrast down/up (8-bit modes)",
+  "left / right chevron \t brightness down/up (8-bit modes)",
+  "asterisk \t\t reset contrast and brightness",
+  "z \t\t\t toggle zoomed mode",
+  "r \t\t\t rotate clockwise 90 degrees",
+  "m \t\t\t mirror",
+  "f \t\t\t flip",
+  "c \t\t\t toggle centering picture on screen",
 #ifdef HANDS_ON_VC_CODE
-  "F1 to F8 \t\t change virtual console\0",
+  "F1 to F8 \t\t change virtual console",
 #endif
-  "Esc or x \t\t exit the viewer\0",
-  " \0",
-  "hjkl or qaop keys scroll around the picture\0",
-  "HJKL or QAOP (also the cursor keys) scroll in bigger steps\0",
-  "\0"
+  "Esc or x \t\t exit the viewer",
+  " ",
+  "hjkl or qaop keys scroll around the picture",
+  "HJKL or QAOP (also the cursor keys) scroll in bigger steps",
+  ""	/* end */
   };
 
 
@@ -72,6 +76,7 @@ struct {
 
 byte *theimage;   /* quite slow if passed as parameter */
 int width,height,numcols; /* same here */
+byte *image_palette;
 
 double contrast=1.0;  /* note that double contrast is in fact 2 :) */
 int brightness=0;
@@ -81,6 +86,8 @@ int palr[256],palg[256],palb[256];
 byte palr64[256],palg64[256],palb64[256];
 int palrgb[768];
 
+int loading_file_type;
+
 int ttyfd;
 
 
@@ -88,65 +95,96 @@ int ttyfd;
 #define GET16BITCOLOUR(r,g,b) (((r>>3)<<11)|((g>>2)<<5)|(b>>3))
 
 
-int readgiforjpg(giffn,howfarfunc)
+
+/* to read a picture only, set show_dont_tell=0.
+ * If you're only reading the picture, not showing it, then:
+ * 1. picture is returned in global vars. 'theimage' and 'image_palette'
+ * 2. you have to set pixelsize=whatever and restore it afterwards.
+ * 3. you have to free() both theimage and image_palette when finished.
+ */
+int readpicture(giffn,howfarfunc,show_dont_tell)
 char *giffn;
 hffunc howfarfunc;
+int show_dont_tell;
 {
 int result;
-GIFINFO ginfo;
+PICINFO ginfo;
 byte *palette;
 
-/* at this stage, we know it ends in either .gif or .jpg */
-/* until I get around to adding PPM support, that is... :) */
-if(!strcmp(strrchr(giffn,'.'),".jpg"))
+
+ginfo.numcols=256;	/* only changes for GIF files */
+
+switch(loading_file_type=magic_ident(giffn))
+  {
+  case _IS_GIF:
+    /* part of the New Strategy (tm); always use 8-bit for GIFs.
+     * fix the mode to a similar 8-bit one if possible.
+     */
+    if(show_dont_tell) fix_to_similar_mode(FIX_TO_EIGHT_BIT);
+    pixelsize=1;
+    result=readgif(giffn,&theimage,&palette,howfarfunc);
+    if(result==_PIC_OK)
+      {
+      getgifinfo(&ginfo);
+      height=ginfo.height; width=ginfo.width;
+      }
+    break;
+
+  case _IS_JPEG:
+    /* don't *always* use 24-bit for JPEGs... check for 15/16/24-bit
+     * video modes before doing so, as well as the config.
+     */
+    if(show_dont_tell)
+      {
+      pixelsize=has_highcolour_mode()?3:1;
+      if(pixelsize==3 && cfg.jpeg24bit==0)
+        pixelsize=1;
+      }
+
+    theimage=NULL;
+    result=read_JPEG_file(giffn,howfarfunc,&palette);
+    
+    /* if error, then palette has already been nuked */
+    if(theimage==NULL)
+      result=_PICERR_NOMEM;
+    else
+      if(result!=_PIC_OK) free(theimage);
+      
+    if(result!=_PIC_OK) result=_PICERR_SHOWN_ALREADY;
+    break;
+    
+  case _IS_PNM:
+    if(show_dont_tell) pixelsize=has_highcolour_mode()?3:1;
+    result=read_pnm_file(giffn,howfarfunc,&theimage,&palette,&pixelsize,
+    		&ginfo);
+    width=ginfo.width;
+    height=ginfo.height;
+    if(result!=_PIC_OK)
+      {
+      if(theimage!=NULL) free(theimage);
+      if(palette!=NULL)  free(palette);
+      }
+  }
+
+
+if(show_dont_tell)
   {
-  /* don't *always* use 24-bit for JPEGs... check for 15/16/24-bit
-   * video modes before doing so.
-   */
-  pixelsize=has_highcolour_mode()?3:1;
   if(pixelsize==3)
     fix_to_similar_mode(FIX_TO_HIGHCOLOUR);
   else
     fix_to_similar_mode(FIX_TO_EIGHT_BIT);
-  isjpeg=1;
-  ginfo.numcols=256;
-  theimage=NULL;
-  result=read_JPEG_file(giffn,howfarfunc,&palette);
-  /* if error, then palette has already been nuked */
-  if(theimage==NULL)
-    result=0;
-  else
-    if(result==0) free(theimage);
-  }
-else
-  {
-  /* part of the New Strategy (tm); always use 8-bit for GIFs.
-   * fix the mode to a similar 8-bit one if possible.
-   */
-  fix_to_similar_mode(FIX_TO_EIGHT_BIT);
-  pixelsize=1;
-  isjpeg=0;
-  result=readgif(giffn,&theimage,&palette,howfarfunc);
-  if(result==_GIF_OK)
+
+  if(result==_PIC_OK)
     {
-    getgifinfo(&ginfo);
-    height=ginfo.height; width=ginfo.width;
+    numcols=ginfo.numcols;
+    showgif(palette);
+    free(theimage);
+    free(palette);
     }
   }
+else
+  image_palette=palette;
 
-if(((isjpeg)&&(result==1))||((!isjpeg)&&(result==_GIF_OK)))
-  {
-  numcols=ginfo.numcols;
-  showgif(palette);
-  free(theimage);
-  free(palette);
-  }
-/* when is a result not a result? */
-if(isjpeg)
-  {
-  result=!result;
-  if(result!=0) result=_GIFERR_SHOWN_ALREADY;
-  }
 return(result);
 }
 
@@ -156,6 +194,8 @@ int has_highcolour_mode()
 {
 int try;
 
+if(cfg.hicolmodes) return(1);	/* forces positive response */
+
 try=modematch(-1,-1,15);
 if(try!=-1)
   if(cfg.mode_allowed[try])
@@ -195,6 +235,7 @@ newmode=-1; pixdiff=(1<<30);
 numpixels=get_mode_width(curvgamode)*get_mode_height(curvgamode);
 
 /* we check 0 - 255 at the most */
+/* Harm's put something in to deal with this, must check that sometime */
 #ifdef HARM_HANEMAAYER_FIXES_THAT_SVGALIB_BUG
 for(f=0;f<256;f++)
 #else
@@ -236,6 +277,22 @@ if((curvgamode==7)||(curvgamode==8)) virtual=1; else virtual=0;
 }
 
 
+aborted_file_cleanup()
+{
+switch(loading_file_type)
+  {
+  case _IS_GIF:
+    aborted_file_gif_cleanup();
+    break;
+  case _IS_JPEG:
+    aborted_file_jpeg_cleanup();
+    break;
+  case _IS_PNM:
+    aborted_file_pnm_cleanup();
+  }
+}
+
+
 is_this_file_jpeg()
 {
 return(isjpeg);
@@ -335,7 +392,6 @@ graphicson()
 {
 int x;
 
-/* workaround for svgalib bug */
 if((vga_hasmode(curvgamode))&&(cfg.mode_allowed[curvgamode]))
   vga_setmode(curvgamode);
 else
@@ -362,8 +418,6 @@ else
     }
   }
 
-/* workaround for screen clearing probs in svgalib 0.95+ */
-vga_clear();
 if(pixelsize!=1) gl_setcontextvga(curvgamode);
 
 scrnwide=get_mode_width(curvgamode);
@@ -387,7 +441,6 @@ if(vkludge)
 graphicsoff()
 {
 /* we let zgv.c deal with this now, so this is just a placeholder */
-/* vga_setmode(TEXT); */
 }
 
 
@@ -403,16 +456,16 @@ int opx,opy,px,py,quitshow,key,redraw;
 if(numcols<256)
   {
   int f;
-  for(f=numcols;f<numcols+64;f++) palr64[f]=palg64[f]=palb64[f]=f-numcols;
+  for(f=numcols;f<numcols+64;f++)
+    palette[f]=palette[256+f]=palette[512+f]=((f-numcols)<<2);
   }
 
-vga_setflipchar('#');
 px=py=quitshow=0;
 filterpal(palette);
 graphicson();
-if((ttyfd=open("/dev/tty0",O_RDONLY|O_NONBLOCK))<0)
+if((ttyfd=open("/dev/tty",O_RDONLY|O_NONBLOCK))<0)
   {graphicsoff();
-  printf("Couldn't get nonblocking read access to /dev/tty0.\n");
+  printf("Couldn't get nonblocking read access to /dev/tty.\n");
   exit(1);}  
 redrawgif(px,py);
 
@@ -493,6 +546,9 @@ while(!quitshow)
       case 'r': fx_rot();    px=py=0; redraw=1; break;
       case 'z':
         zoom=(!zoom); redraw=1; px=py=0; graphicson(); break;
+      case 'c':
+        cfg.centreflag=(!cfg.centreflag);
+        redraw=1; px=py=0; graphicson(); break;
 #if 0
 /* this appears to be a bit broken with VC switching after ^Z */
       case 26:   /* ^Z */
@@ -535,6 +591,7 @@ redrawgif(px,py)
 int px,py;
 {
 int x,y,xdim;
+int x_add,y_add;
 byte *realline,*ptr;
 long place;
 
@@ -542,17 +599,31 @@ if(zoom)
   drawzoomedgif(height,width,theimage);
 else
   {
+  /* draw non-zoomed pic */
+  x_add=y_add=0;	/* these control the centering */
+  if(cfg.centreflag)
+    {
+    if(width<scrnwide)
+      x_add=(scrnwide-width)>>1;
+    
+    if(height<scrnhigh)
+      y_add=(scrnhigh-height)>>1;
+      
+    if(virtual) x_add>>=1;
+    }
+  
+  
   if(width-px<scrnwide) xdim=width-px; else xdim=scrnwide;
   if((py>=height)||(px>=width)) return(0);
   /* hopefully the following is fairly quick as fewer ifs... ? */
   if((virtual)&&(pixelsize==1))
     {
-    if((realline=malloc(scrnwide))==NULL) return(0);
+    if((realline=calloc(1,scrnwide))==NULL) return(0);
     for(y=0;(y<height-py)&&(y<scrnhigh);y++)
       {
       for(x=0;x<xdim;x++)
         *(realline+(x>>1))=getvpix(px,py,x,y);
-        vga_drawscansegment(realline,0,y,xdim>>1);
+      vga_drawscansegment(realline,x_add,y+y_add,xdim>>1);
       }
     free(realline);
     }
@@ -561,14 +632,33 @@ else
     switch(scrnpixelsize)
       {
       case 3:
-        gl_putboxpart(0,0,(scrnwide<width)?scrnwide:width,
+        gl_putboxpart(x_add,y_add,(scrnwide<width)?scrnwide:width,
 	        (scrnhigh<height)?scrnhigh:height,width,height,theimage,px,py);
         break;
       
       default:
-        for(y=0;(y<height-py)&&(y<scrnhigh);y++)
-          eventuallydrawscansegment(theimage+pixelsize*((py+y)*width+px),
-          	0,y,xdim*pixelsize);
+        /* for 320x200 (I think), 320x240, 320x400, 360x480 we need the
+         * length to be a multiple of 8. :(  We allocate an image-width
+         * line (blanked), and display via that.
+         */
+        if(curvgamode==G320x200x256 || curvgamode==G320x240x256 ||
+           curvgamode==G320x400x256 || curvgamode==G360x480x256)
+          {
+          unsigned char *hack;
+          
+          if((hack=malloc(scrnwide))==NULL) return(0);
+          for(y=0;(y<height-py)&&(y<scrnhigh);y++)
+            {
+            /* assume pixelsize==1, because it must be */
+            memcpy(hack,theimage+((py+y)*width+px),xdim);
+            vga_drawscansegment(hack,x_add,y+y_add,xdim);
+            }
+          free(hack);	/* bit like zgv, really :) */
+          }
+        else
+          for(y=0;(y<height-py)&&(y<scrnhigh);y++)
+            eventuallydrawscansegment(theimage+pixelsize*((py+y)*width+px),
+            	x_add,y+y_add,xdim*pixelsize);
         break;
       }
     }
@@ -627,6 +717,7 @@ drawzoomedgif()
 register int a,b,x,yp,yw;
 long y,sw,sh,lastyp;
 int xoff,yoff;
+int x_add,y_add;	/* for centering */
 int c,cr,cg,cb,pixwide,pixhigh,pr,pg,pb;
 long place;
 int bigimage;
@@ -644,6 +735,20 @@ if(sh>scrnhigh)
   { sh=scrnhigh; sw=(int)((scrnhigh*((long)width))/((long)height)); }
 
 /* so now our zoomed image will be sw x sh */
+
+/* fix centering if needed */
+x_add=y_add=0;
+if(cfg.centreflag)
+  {
+  if(sw<scrnwide)
+    x_add=(scrnwide-sw)>>1;
+  
+  if(sh<scrnhigh)
+    y_add=(scrnhigh-sh)>>1;
+  
+  if(virtual) x_add>>=1;
+  }
+
 bigimage=(width>sw)?1:0;   /* 1 if image has been reduced, 0 if made bigger */
 if(bigimage)
   /* it's been reduced - easy, just make 'em fit in less space */
@@ -704,7 +809,7 @@ if(bigimage)
           else
             memcpy(rline+(x*sw)/width*3,theimage+yw*3,3);
           }
-      eventuallydrawscansegment(rline,0,yp,scrnwide*pixelsize);
+      eventuallydrawscansegment(rline,x_add,yp+y_add,scrnwide*pixelsize);
       lastyp=yp;
       }
     }
@@ -739,8 +844,8 @@ else
         cb=*(theimage+yw); cg=*(theimage+yw+1); cr=*(theimage+yw+2);
         }
       xoff=(x*sw)/width;
-      for(b=0;b<pixhigh;b++)
-        for(a=0;a<pixwide;a++)
+      for(b=0;b<pixhigh && yoff+b<scrnhigh;b++)
+        for(a=0;a<pixwide && xoff+a<scrnwide;a++)
           if(scrncols<=256)
             *(rline+(yoff+b)*scrnwide+xoff+a)=c;
           else
@@ -752,11 +857,11 @@ else
             }
       }
     eventuallydrawscansegment(rline+pixelsize*y*scrnwide,
-    	0,y,scrnwide*pixelsize); /* really pessimistic */
+    	x_add,y+y_add,scrnwide*pixelsize); /* really pessimistic */
     }
   for(;y<scrnhigh;y++)          /* and do the rest */
     eventuallydrawscansegment(rline+pixelsize*y*scrnwide,
-    				0,y,scrnwide*pixelsize);
+    				x_add,y+y_add,scrnwide*pixelsize);
   free(rline);
   }
 }
