@@ -10,5 +10,9 @@
 extern int curvgamode,zoom,virtual,vkludge,brightness,pixelsize;
 extern double contrast;
 
-extern int readgiforjpg(char *,hffunc);
+/* reqd. by zgv.c:makexv332() */
+extern unsigned char *theimage,*image_palette;
+extern int width,height;
+
+extern int readpicture(char *,hffunc,int);
 extern int is_this_file_jpeg();
